package com.ozs.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.validation.Validator;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.PageHelper;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysUserUpdate;
import com.ozs.common.core.domain.entity.SysUsers;
import com.ozs.common.core.domain.vo.SysRoleRequestVo;
import com.ozs.common.core.domain.vo.SysUserVo;
import com.ozs.common.utils.uuid.IdUtils;
import com.ozs.system.mapper.SysDeptMapper;
import icu.mhb.mybatisplus.plugln.base.service.impl.JoinServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.ozs.common.annotation.DataScope;
import com.ozs.common.constant.UserConstants;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.exception.ServiceException;
import com.ozs.common.utils.SecurityUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.bean.BeanValidators;
import com.ozs.common.utils.spring.SpringUtils;
import com.ozs.system.domain.SysPost;
import com.ozs.system.domain.SysUserPost;
import com.ozs.system.domain.SysUserRole;
import com.ozs.system.mapper.SysPostMapper;
import com.ozs.system.mapper.SysRoleMapper;
import com.ozs.system.mapper.SysUserMapper;
import com.ozs.system.mapper.SysUserPostMapper;
import com.ozs.system.mapper.SysUserRoleMapper;
import com.ozs.system.service.ISysConfigService;
import com.ozs.system.service.ISysUserService;
import org.springframework.util.ObjectUtils;

/**
 * 用户 业务层处理
 *
 * @author hx
 */
@Service
public class SysUserServiceImpl extends JoinServiceImpl<SysUserMapper, SysUser> implements ISysUserService {
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    protected Validator validator;
    @Autowired
    private SysDeptMapper deptMapper;

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(SysUser user) {
        return userMapper.selectUserList(user);
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectAllocatedList(SysUser user) {
        return userMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUnallocatedList(SysUser user) {
        return userMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName) {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param id 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long id) {
        return userMapper.selectUserById(id);
    }

    /**
     * 查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName) {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list)) {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName) {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        if (CollectionUtils.isEmpty(list)) {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public String checkUserNameUnique(SysUser user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public String checkUserNameUniqueUpdate(SysUserUpdate user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkPhoneUnique(SysUser user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhoneNum());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkPhoneUniqueUpdate(SysUserUpdate user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhoneNum());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkEmailUnique(SysUser user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkEmailUniqueUpdate(SysUserUpdate user) {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user) {
        if (StringUtils.isNotNull(user.getId()) && user.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowedUpdate(SysUserUpdate user) {
        if (StringUtils.isNotNull(user.getId()) && user.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     *
     * @param id 用户id
     */
    @Override
    public void checkUserDataScope(Long id) {
        if (!SysUser.isAdmin(SecurityUtils.getId())) {
            SysUser user = new SysUser();
            user.setId(id);
            List<SysUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
            if (StringUtils.isEmpty(users)) {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(SysUser user) {
        // 新增用户信息
        int rows = userMapper.insertUser(user);
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(SysUser user) {
        return userMapper.insertUser(user) > 0;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(SysUser user) {
        Long userId = user.getId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(user);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPostByUserId(userId);
        // 新增用户与岗位管理
        insertUserPost(user);
        return userMapper.updateUser(user);
    }

    /**
     * 用户授权角色
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(Long userId, Long[] roleIds) {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user) {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user) {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar   头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar) {
        return userMapper.updateUserAvatar(userName, avatar) > 0;
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user) {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password) {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user) {
        this.insertUserRole(user.getId(), user.getRoleIds());
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRoleUpdate(SysUserUpdate user) {
        this.insertUserRole(user.getId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     *
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user) {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty(posts)) {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
            for (Long postId : posts) {
                SysUserPost up = new SysUserPost();
                up.setId(user.getId());
                up.setPostId(postId);
                list.add(up);
            }
            userPostMapper.batchUserPost(list);
        }
    }

    /**
     * 新增用户角色信息
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds) {
        if (StringUtils.isNotEmpty(roleIds)) {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
            for (Long roleId : roleIds) {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

    /**
     * 通过用户ID删除用户
     *
     * @param id 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long id) {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(id);
        // 删除用户与岗位表
        userPostMapper.deleteUserPostByUserId(id);
        return userMapper.deleteUserById(id);
    }

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds) {
//        for (Long userId : userIds) {
//          checkUserAllowed(new SysUser(userId));
//            checkUserDataScope(userId);
//        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        return userMapper.deleteUserByIds(userIds);
    }

    /**
     * 导入用户数据
     *
     * @param userList        用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName        操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<SysUsers> userList, Boolean isUpdateSupport, String operName,String dictValue) {
        if (StringUtils.isNull(userList) || userList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        StringBuilder errMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");
        for (SysUsers user : userList) {
            try {
                // 验证是否存在这个用户
                SysUser sysUser = new SysUser();
                BeanUtils.copyProperties(user, sysUser);
                SysUser u = userMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull(u)) {
                    if (!ObjectUtils.isEmpty(user.getPhoneNum())
                            && UserConstants.NOT_UNIQUE.equals(checkPhoneUnique(sysUser))) {
                        errMsg.append("手机号码:" + user.getPhoneNum() + "已存在");
                        int a=10/0;
                    }else if (!ObjectUtils.isEmpty(user.getEmail())
                            && UserConstants.NOT_UNIQUE.equals(checkEmailUnique(sysUser))) {
                        errMsg.append("邮箱:" + user.getEmail() + "已存在");
                        int a=10/0;
                    }
                    BeanValidators.validateWithException(validator, user);
                    sysUser.setPassword(SecurityUtils.encryptPassword(password));
                    sysUser.setCreateBy(operName);
                    sysUser.setUpdateBy(operName);
                    sysUser.setUserId(IdUtils.fastSimpleUUID());
                    SysDept sysDept = deptMapper.selectDeptByDeptName(user.getDeptName());
                    if (ObjectUtils.isEmpty(sysDept)) {
                        errMsg.append("部门名称:" + user.getDeptName() + "不存在");
                    }
                    sysUser.setDeptId(sysDept.getDeptId());
                    sysUser.setPassword(SecurityUtils.encryptPassword(dictValue));
                    this.insertUser(sysUser);
                    SysUser sysUser1 = userMapper.selectUserByUserName(user.getUserName());
                    String roleName = sysUser.getRoleName();
                    String[] split = roleName.split(",");
                    if (StringUtils.isNotEmpty(split)) {
                        List<Long> id = new ArrayList<>();
                        for (int i = 0; i < split.length; i++) {
                            Long roleId = roleMapper.getRoleId(split[i]);
                            List<SysUserRole> list = new ArrayList<SysUserRole>(id.size());
                            SysUserRole ur = new SysUserRole();
                            ur.setUserId(sysUser1.getId());
                            ur.setRoleId(roleId);
                            list.add(ur);
                            userRoleMapper.batchUserRole(list);
                        }
                    }
                    successNum++;
                    successMsg.append(successNum + "、账号 " + user.getUserName() + " 导入成功");
                } else if (isUpdateSupport) {
                    BeanValidators.validateWithException(validator, user);
//                    checkUserAllowed(sysUser);
//                    checkUserDataScope(sysUser.getId());
                    sysUser.setUpdateBy(operName);
                    SysDept sysDept = deptMapper.selectDeptByDeptName(user.getDeptName());
                    sysUser.setDeptId(sysDept.getDeptId());
                    this.updateUser(sysUser);
                    successNum++;
                    successMsg.append(successNum + "、账号 " + user.getUserName() + " 更新成功");
                } else {
                    failureNum++;
                    failureMsg.append(failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            } catch (Exception e) {
                failureNum++;
                String msg = failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                if (errMsg.length() > 0) {
                    failureMsg.append(msg + errMsg.toString());
                    errMsg.setLength(0);
                } else {
                    failureMsg.append(msg + e.getMessage());
                }
                log.error(msg, e);
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    @Override
    public int saveUser(SysUser user) {
        // 新增用户信息
        user.setUserId(IdUtils.fastSimpleUUID());
        int rows = userMapper.insertUser(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    @Override
    public int updateUsers(SysUser user) {
        Long userId = user.getId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(user);
        user.setUserId(IdUtils.fastSimpleUUID());
        return userMapper.updateUser(user);
    }

    @Override
    public int updateUsersUpdate(SysUserUpdate user) {
        Long userId = user.getId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRoleUpdate(user);
        user.setUserId(IdUtils.fastSimpleUUID());
        return userMapper.updateUserUpdate(user);
    }

    /**
     * 修改用户头像
     *
     * @param userId 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatarById(String userId, String avatar) {
        return userMapper.updateUserAvatarById(userId, avatar);
    }

    /**
     * 修改用户密码
     *
     * @param userId 用户名
     * @param newPwd 新密码
     * @return 结果
     */
    @Override
    public boolean updatePassword(String userId, String newPwd) {
        return userMapper.updatePassword(userId, newPwd);
    }

    @Override
    public SysUser getUserByUserName(String username) {
        return userMapper.selectUserByUserName(username);
    }

    @Override
    public boolean dataPermission(Long id, Integer dataPermission) {
        return userMapper.dataPermission(id, dataPermission);
    }

    @Override
    public IPage<SysUser> queryPage(SysUserVo sysUserVo) {
        int pageNum = Integer.parseInt(sysUserVo.getPageNum().toString());
        int pageSize = Integer.parseInt(sysUserVo.getPageSize().toString());
        com.github.pagehelper.Page<SysUser> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> userMapper.selectUserLists(sysUserVo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SysUser> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public List<SysUser> usetList(SysUserVo sysUserVo) {
        return userMapper.selectUserLists(sysUserVo);
    }
}
